@@ -84,6 +84,7 @@
 #include "frame.h"
 #include "colormaps.h"
 #include "decorations.h"
+#include "functions.h"
 
 /* ---------------------------- local definitions -------------------------- */
 
@@ -446,7 +447,7 @@ static void setup_window_structure(
 			*pfw, USED_TITLE_DIR_FOR_SHADING(savewin));
 		SET_SHADED_DIR(*pfw, SHADED_DIR(savewin));
 		SET_NAME_CHANGED(*pfw,IS_NAME_CHANGED(savewin));
-		SET_PLACED_WB3(*pfw,IS_PLACED_WB3(savewin));
+		(*pfw)->placed_by_button = savewin->placed_by_button;
 		SET_PLACED_BY_FVWM(*pfw, IS_PLACED_BY_FVWM(savewin));
 		SET_HAS_EWMH_WM_ICON_HINT(*pfw, HAS_EWMH_WM_ICON_HINT(savewin));
 		(*pfw)->ewmh_mini_icon_width = savewin->ewmh_mini_icon_width;
@@ -855,7 +856,7 @@ static void setup_button_windows(
 	pattributes->cursor = Scr.FvwmCursors[CRS_SYS];
 	pattributes->event_mask = XEVMASK_BUTTONW;
 
-	for (i = 0; i < NUMBER_OF_BUTTONS; i++)
+	for (i = 0; i < NUMBER_OF_TITLE_BUTTONS; i++)
 	{
 		has_button = (((!(i & 1) && i / 2 < Scr.nr_left_buttons) ||
 			       ( (i & 1) && i / 2 < Scr.nr_right_buttons)) &&
@@ -893,7 +894,7 @@ static void destroy_button_windows(FvwmWindow *fw, Bool do_only_delete_context)
 	int i;
 	Bool is_deleted = False;
 
-	for (i = 0; i < NUMBER_OF_BUTTONS; i++)
+	for (i = 0; i < NUMBER_OF_TITLE_BUTTONS; i++)
 	{
 		if (FW_W_BUTTON(fw, i) != None)
 		{
@@ -937,9 +938,10 @@ static void setup_parent_window(FvwmWindow *fw)
 	size_borders b;
 
 	XSetWindowAttributes attributes;
-	int valuemask = CWBackingStore | CWBackPixmap | CWCursor | CWEventMask |
-		CWSaveUnder;
+	int valuemask;
 
+	valuemask = CWBackingStore | CWBackPixmap | CWCursor | CWEventMask |
+		CWSaveUnder;
 	attributes.backing_store = NotUseful;
 	attributes.background_pixmap = None;
 	attributes.cursor = Scr.FvwmCursors[CRS_DEFAULT];
@@ -1087,7 +1089,7 @@ static void setup_frame_stacking(FvwmWindow *fw)
 {
 	int i;
 	int n;
-	Window w[10 + NUMBER_OF_BUTTONS];
+	Window w[10 + NUMBER_OF_TITLE_BUTTONS];
 
 	/* Stacking order (top to bottom):
 	 *  - Parent window
@@ -1103,7 +1105,7 @@ static void setup_frame_stacking(FvwmWindow *fw)
 	}
 	if (HAS_TITLE(fw))
 	{
-		for (i = 0; i < NUMBER_OF_BUTTONS; i += 2)
+		for (i = 0; i < NUMBER_OF_TITLE_BUTTONS; i += 2)
 		{
 			if (FW_W_BUTTON(fw, i) != None)
 			{
@@ -1829,7 +1831,6 @@ void setup_placement_penalty(FvwmWindow *fw, window_style *pstyle)
 void setup_frame_attributes(
 	FvwmWindow *fw, window_style *pstyle)
 {
-	int i;
 	XSetWindowAttributes xswa;
 
 	/* Backing_store is controlled on the client, borders, title & buttons
@@ -1847,36 +1848,18 @@ void setup_frame_attributes(
 		xswa.backing_store = NotUseful;
 		break;
 	}
-	XChangeWindowAttributes(dpy, FW_W(fw), CWBackingStore, &xswa);
-	if (pstyle->flags.use_backing_store == BACKINGSTORE_OFF)
-	{
-		xswa.backing_store = NotUseful;
-	}
-	if (HAS_TITLE(fw))
-	{
-		XChangeWindowAttributes(
-			dpy, FW_W_TITLE(fw), CWBackingStore, &xswa);
-		for (i = 0; i < NUMBER_OF_BUTTONS; i++)
-		{
-			if (FW_W_BUTTON(fw, i))
-			{
-				XChangeWindowAttributes(
-					dpy, FW_W_BUTTON(fw, i),
-					CWBackingStore, &xswa);
-			}
-		}
-	}
-
 	/* parent_relative is applied to the frame and the parent */
 	xswa.background_pixmap = pstyle->flags.use_parent_relative
 		? ParentRelative : None;
-	XChangeWindowAttributes(dpy, FW_W_FRAME(fw), CWBackPixmap, &xswa);
-	XChangeWindowAttributes(dpy, FW_W_PARENT(fw), CWBackPixmap, &xswa);
-
 	/* Save_under is only useful on the frame */
 	xswa.save_under = pstyle->flags.do_save_under
 		? Scr.flags.do_save_under : NotUseful;
-	XChangeWindowAttributes(dpy, FW_W_FRAME(fw), CWSaveUnder, &xswa);
+	XChangeWindowAttributes(dpy, FW_W(fw), CWBackingStore, &xswa);
+	XChangeWindowAttributes(
+		dpy, FW_W_PARENT(fw), CWBackPixmap | CWBackingStore, &xswa);
+	XChangeWindowAttributes(
+		dpy, FW_W_FRAME(fw),
+		CWBackPixmap | CWBackingStore | CWSaveUnder, &xswa);
 
 	return;
 }
@@ -1953,121 +1936,6 @@ void setup_focus_policy(FvwmWindow *fw)
 	return;
 }
 
-Bool setup_window_placement(
-	FvwmWindow *fw, window_style *pstyle, rectangle *attr_g,
-	initial_window_options_t *win_opts)
-{
-	int client_argc = 0;
-	char **client_argv = NULL;
-	XrmValue rm_value;
-	int tmpno1 = -1, tmpno2 = -1, tmpno3 = -1, spargs = 0;
-	/* Used to parse command line of clients for specific desk requests. */
-	/* Todo: check for multiple desks. */
-	XrmDatabase db = NULL;
-	static XrmOptionDescRec table [] = {
-		/* Want to accept "-workspace N" or -xrm "fvwm*desk:N" as
-		 * options to specify the desktop. I have to include dummy
-		 * options that are meaningless since Xrm seems to allow -w to
-		 * match -workspace if there would be no ambiguity. */
-		{"-workspacf", "*junk", XrmoptionSepArg, (caddr_t) NULL},
-		{"-workspace", "*desk", XrmoptionSepArg, (caddr_t) NULL},
-		{"-xrn", NULL, XrmoptionResArg, (caddr_t) NULL},
-		{"-xrm", NULL, XrmoptionResArg, (caddr_t) NULL},
-	};
-	Bool rc;
-	const exec_context_t *exc;
-	exec_context_changes_t ecc;
-
-	/* Find out if the client requested a specific desk on the command
-	 * line. */
-	/*  RBW - 11/20/1998 - allow a desk of -1 to work.  */
-
-	if (XGetCommand(dpy, FW_W(fw), &client_argv, &client_argc) &&
-	    client_argc > 0 && client_argv != NULL)
-	{
-		/* Get global X resources */
-		MergeXResources(dpy, &db, False);
-
-		/* command line takes precedence over all */
-		MergeCmdLineResources(
-			&db, table, 4, client_argv[0], &client_argc,
-			client_argv, True);
-
-		/* Now parse the database values: */
-		if (GetResourceString(db, "desk", client_argv[0], &rm_value) &&
-		    rm_value.size != 0)
-		{
-			SGET_START_DESK(*pstyle) = atoi(rm_value.addr);
-			/*  RBW - 11/20/1998  */
-			if (SGET_START_DESK(*pstyle) > -1)
-			{
-				SSET_START_DESK(
-					*pstyle, SGET_START_DESK(*pstyle) + 1);
-			}
-			pstyle->flags.use_start_on_desk = 1;
-		}
-		if (GetResourceString(
-			    db, "fvwmscreen", client_argv[0], &rm_value) &&
-		    rm_value.size != 0)
-		{
-			SSET_START_SCREEN(
-				*pstyle, FScreenGetScreenArgument(
-					rm_value.addr, 'c'));
-			pstyle->flags.use_start_on_screen = 1;
-		}
-		if (GetResourceString(db, "page", client_argv[0], &rm_value) &&
-		    rm_value.size != 0)
-		{
-			spargs = sscanf(
-				rm_value.addr, "%d %d %d", &tmpno1, &tmpno2,
-				&tmpno3);
-			switch (spargs)
-			{
-			case 1:
-				pstyle->flags.use_start_on_desk = 1;
-				SSET_START_DESK(*pstyle, (tmpno1 > -1) ?
-						tmpno1 + 1 : tmpno1);
-				break;
-			case 2:
-				pstyle->flags.use_start_on_desk = 1;
-				SSET_START_PAGE_X(*pstyle, (tmpno1 > -1) ?
-						  tmpno1 + 1 : tmpno1);
-				SSET_START_PAGE_Y(*pstyle, (tmpno2 > -1) ?
-						  tmpno2 + 1 : tmpno2);
-				break;
-			case 3:
-				pstyle->flags.use_start_on_desk = 1;
-				SSET_START_DESK(*pstyle, (tmpno1 > -1) ?
-						tmpno1 + 1 : tmpno1);
-				SSET_START_PAGE_X(*pstyle, (tmpno2 > -1) ?
-						  tmpno2 + 1 : tmpno2);
-				SSET_START_PAGE_Y(*pstyle, (tmpno3 > -1) ?
-						  tmpno3 + 1 : tmpno3);
-				break;
-			default:
-				break;
-			}
-		}
-
-		XFreeStringList(client_argv);
-		XrmDestroyDatabase(db);
-	}
-	if (pstyle->flags.do_start_iconic)
-	{
-		win_opts->initial_state = IconicState;
-	}
-	ecc.type = EXCT_NULL;
-	ecc.w.fw = fw;
-	exc = exc_create_context(&ecc, ECC_TYPE | ECC_FW);
-	rc = PlaceWindow(
-		exc, &pstyle->flags, attr_g, SGET_START_DESK(*pstyle),
-		SGET_START_PAGE_X(*pstyle), SGET_START_PAGE_Y(*pstyle),
-		SGET_START_SCREEN(*pstyle), PLACE_INITIAL, win_opts);
-	exc_destroy_context(exc);
-
-	return rc;
-}
-
 Bool validate_transientfor(FvwmWindow *fw)
 {
 	XWindowAttributes wa;
@@ -2161,7 +2029,7 @@ FvwmWindow *AddWindow(
 		    dpy, w, &JunkRoot, &JunkX, &JunkY, &JunkWidth, &JunkHeight,
 		    &JunkBW,  &JunkDepth) == 0)
 	{
-		if (Scr.bo.DisplayNewWindowNames)
+		if (Scr.bo.do_display_new_window_names)
 		{
 			fvwm_msg(INFO, "AddWindow", "new window disappeared");
 		}
@@ -2180,7 +2048,7 @@ FvwmWindow *AddWindow(
 	sflags = SGET_FLAGS_POINTER(style);
 	if (SIS_UNMANAGED(sflags))
 	{
-		if (Scr.bo.DisplayNewWindowNames)
+		if (Scr.bo.do_display_new_window_names)
 		{
 			fvwm_msg(
 				INFO, "AddWindow", "new window is unmanaged:\n"
@@ -2220,7 +2088,7 @@ FvwmWindow *AddWindow(
 	/***** visible window name ****/
 	setup_visible_name(fw, False);
 	EWMH_SetVisibleName(fw, False);
-	if (Scr.bo.DisplayNewWindowNames)
+	if (Scr.bo.do_display_new_window_names)
 	{
 		fvwm_msg(
 			INFO, "AddWindow", "new window:\n"
@@ -2347,7 +2215,7 @@ FvwmWindow *AddWindow(
 		attr_g.width = wattr.width;
 		attr_g.height = wattr.height;
 		do_resize_too = setup_window_placement(
-			fw, &style, &attr_g, win_opts);
+			fw, &style, &attr_g, win_opts, PLACE_INITIAL);
 		wattr.x = attr_g.x;
 		wattr.y = attr_g.y;
 
@@ -2488,6 +2356,7 @@ FvwmWindow *AddWindow(
 	{
 		XEvent e;
 
+		memset(&e, 0, sizeof(e));
 		FWarpPointer(
 			dpy, Scr.Root, Scr.Root, 0, 0, Scr.MyDisplayWidth,
 			Scr.MyDisplayHeight,
@@ -2495,6 +2364,7 @@ FvwmWindow *AddWindow(
 			fw->frame_g.y + (fw->frame_g.height>>1));
 		e.xany.type = ButtonPress;
 		e.xbutton.button = 1;
+		e.xbutton.state = Button1Mask;
 		e.xbutton.x_root = fw->frame_g.x + (fw->frame_g.width>>1);
 		e.xbutton.y_root = fw->frame_g.y + (fw->frame_g.height>>1);
 		e.xbutton.x = (fw->frame_g.width>>1);
@@ -2556,6 +2426,28 @@ FvwmWindow *AddWindow(
 			fw, fw->frame_g.x, fw->frame_g.y, fw->frame_g.width,
 			fw->frame_g.height, 0, False);
 	}
+	if (HAS_EWMH_INIT_MAXVERT_STATE(fw) == EWMH_STATE_HAS_HINT ||
+	    HAS_EWMH_INIT_MAXHORIZ_STATE(fw) == EWMH_STATE_HAS_HINT)
+	{
+		int h;
+		int v;
+		char cmd[256];
+
+		if (is_function_allowed(F_MAXIMIZE, NULL, fw, True, False))
+		{
+			h = (HAS_EWMH_INIT_MAXHORIZ_STATE(fw) ==
+			     EWMH_STATE_HAS_HINT) ? 100 : 0;
+			v = (HAS_EWMH_INIT_MAXVERT_STATE(fw) ==
+			     EWMH_STATE_HAS_HINT) ? 100 : 0;
+			sprintf(cmd,"Maximize on %i %i", h, v);
+			execute_function_override_window(
+				NULL, NULL, cmd, 0, fw);
+		}
+	}
+	if (HAS_EWMH_INIT_FULLSCREEN_STATE(fw) == EWMH_STATE_HAS_HINT)
+	{
+		EWMH_fullscreen(fw);
+	}
 	if (!XGetGeometry(dpy, FW_W(fw), &JunkRoot, &JunkX, &JunkY, &JunkWidth,
 			  &JunkHeight, &JunkBW,  &JunkDepth))
 	{
@@ -2582,6 +2474,7 @@ FvwmWindow *AddWindow(
 void FetchWmProtocols(FvwmWindow *tmp)
 {
 	Atom *protocols = NULL, *ap;
+	long *l_protocols;
 	int i, n;
 	Atom atype;
 	int aformat;
@@ -2620,10 +2513,11 @@ void FetchWmProtocols(FvwmWindow *tmp)
 			     dpy, FW_W(tmp), _XA_WM_PROTOCOLS, 0L, 10L, False,
 			     _XA_WM_PROTOCOLS, &atype, &aformat, &nitems,
 			     &bytes_remain,
-			     (unsigned char **)&protocols)) == Success)
+			     (unsigned char **)&l_protocols)) == Success)
 		{
-			for (i = 0, ap = protocols; i < nitems; i++, ap++)
+			for (i = 0; i < nitems; i++)
 			{
+				ap = &(l_protocols[i]);
 				if (*ap == (Atom)_XA_WM_TAKE_FOCUS)
 				{
 					SET_WM_TAKES_FOCUS(tmp, 1);
@@ -2656,111 +2550,111 @@ void FetchWmProtocols(FvwmWindow *tmp)
  *
  */
 
-void GetWindowSizeHints(FvwmWindow *tmp)
+void GetWindowSizeHints(FvwmWindow *fw)
 {
 	long supplied = 0;
 	char *broken_cause ="";
 	XSizeHints orig_hints;
 
-	if (HAS_OVERRIDE_SIZE_HINTS(tmp) ||
-	    !XGetWMNormalHints(dpy, FW_W(tmp), &tmp->hints, &supplied))
+	if (HAS_OVERRIDE_SIZE_HINTS(fw) ||
+	    !XGetWMNormalHints(dpy, FW_W(fw), &fw->hints, &supplied))
 	{
-		tmp->hints.flags = 0;
+		fw->hints.flags = 0;
 		memset(&orig_hints, 0, sizeof(orig_hints));
 	}
 	else
 	{
-		memcpy(&orig_hints, &tmp->hints, sizeof(orig_hints));
+		memcpy(&orig_hints, &fw->hints, sizeof(orig_hints));
 	}
 
 	/* Beat up our copy of the hints, so that all important field are
 	 * filled in! */
-	if (tmp->hints.flags & PResizeInc)
+	if (fw->hints.flags & PResizeInc)
 	{
-		SET_SIZE_INC_SET(tmp, 1);
-		if (tmp->hints.width_inc <= 0)
+		SET_SIZE_INC_SET(fw, 1);
+		if (fw->hints.width_inc <= 0)
 		{
-			if (tmp->hints.width_inc < 0 ||
-			    (tmp->hints.width_inc == 0 &&
-			     (tmp->hints.flags & PMinSize) &&
-			     (tmp->hints.flags & PMaxSize) &&
-			     tmp->hints.min_width != tmp->hints.max_width))
+			if (fw->hints.width_inc < 0 ||
+			    (fw->hints.width_inc == 0 &&
+			     (fw->hints.flags & PMinSize) &&
+			     (fw->hints.flags & PMaxSize) &&
+			     fw->hints.min_width != fw->hints.max_width))
 			{
 				broken_cause = "width_inc";
 			}
-			tmp->hints.width_inc = 1;
-			SET_SIZE_INC_SET(tmp, 0);
+			fw->hints.width_inc = 1;
+			SET_SIZE_INC_SET(fw, 0);
 		}
-		if (tmp->hints.height_inc <= 0)
+		if (fw->hints.height_inc <= 0)
 		{
-			if (tmp->hints.height_inc < 0 ||
-			    (tmp->hints.height_inc == 0 &&
-			     (tmp->hints.flags & PMinSize) &&
-			     (tmp->hints.flags & PMaxSize) &&
-			     tmp->hints.min_height != tmp->hints.max_height))
+			if (fw->hints.height_inc < 0 ||
+			    (fw->hints.height_inc == 0 &&
+			     (fw->hints.flags & PMinSize) &&
+			     (fw->hints.flags & PMaxSize) &&
+			     fw->hints.min_height != fw->hints.max_height))
 			{
 				if (!*broken_cause)
 				{
 					broken_cause = "height_inc";
 				}
 			}
-			tmp->hints.height_inc = 1;
-			SET_SIZE_INC_SET(tmp, 0);
+			fw->hints.height_inc = 1;
+			SET_SIZE_INC_SET(fw, 0);
 		}
 	}
 	else
 	{
-		SET_SIZE_INC_SET(tmp, 0);
-		tmp->hints.width_inc = 1;
-		tmp->hints.height_inc = 1;
+		SET_SIZE_INC_SET(fw, 0);
+		fw->hints.width_inc = 1;
+		fw->hints.height_inc = 1;
 	}
 
-	if (tmp->hints.flags & PMinSize)
+	if (fw->hints.flags & PMinSize)
 	{
-		if (tmp->hints.min_width < 0 && !*broken_cause)
+		if (fw->hints.min_width < 0 && !*broken_cause)
 		{
 			broken_cause = "min_width";
 		}
-		if (tmp->hints.min_height < 0 && !*broken_cause)
+		if (fw->hints.min_height < 0 && !*broken_cause)
 		{
 			broken_cause = "min_height";
 		}
 	}
 	else
 	{
-		if (tmp->hints.flags & PBaseSize)
+		if (fw->hints.flags & PBaseSize)
 		{
-			tmp->hints.min_width = tmp->hints.base_width;
-			tmp->hints.min_height = tmp->hints.base_height;
+			fw->hints.min_width = fw->hints.base_width;
+			fw->hints.min_height = fw->hints.base_height;
 		}
 		else
 		{
-			tmp->hints.min_width = 1;
-			tmp->hints.min_height = 1;
+			fw->hints.min_width = 1;
+			fw->hints.min_height = 1;
 		}
 	}
-	if (tmp->hints.min_width <= 0)
+	if (fw->hints.min_width <= 0)
 	{
-		tmp->hints.min_width = 1;
+		fw->hints.min_width = 1;
 	}
-	if (tmp->hints.min_height <= 0)
+	if (fw->hints.min_height <= 0)
 	{
-		tmp->hints.min_height = 1;
+		fw->hints.min_height = 1;
 	}
 
-	if (tmp->hints.flags & PMaxSize)
+	if (fw->hints.flags & PMaxSize)
 	{
-		if (tmp->hints.max_width < tmp->hints.min_width)
+		if (fw->hints.max_width < fw->hints.min_width)
 		{
-			tmp->hints.max_width = DEFAULT_MAX_MAX_WINDOW_WIDTH;
+			fw->hints.max_width = DEFAULT_MAX_MAX_WINDOW_WIDTH;
 			if (!*broken_cause)
 			{
 				broken_cause = "max_width";
 			}
 		}
-		if (tmp->hints.max_height < tmp->hints.min_height)
+		if (fw->hints.max_height < fw->hints.min_height)
 		{
-			tmp->hints.max_height = DEFAULT_MAX_MAX_WINDOW_HEIGHT;
+			fw->hints.max_height = DEFAULT_MAX_MAX_WINDOW_HEIGHT;
 			if (!*broken_cause)
 			{
 				broken_cause = "max_height";
@@ -2769,8 +2663,8 @@ void GetWindowSizeHints(FvwmWindow *tmp)
 	}
 	else
 	{
-		tmp->hints.max_width = DEFAULT_MAX_MAX_WINDOW_WIDTH;
-		tmp->hints.max_height = DEFAULT_MAX_MAX_WINDOW_HEIGHT;
+		fw->hints.max_width = DEFAULT_MAX_MAX_WINDOW_WIDTH;
+		fw->hints.max_height = DEFAULT_MAX_MAX_WINDOW_HEIGHT;
 	}
 
 	/*
@@ -2778,26 +2672,26 @@ void GetWindowSizeHints(FvwmWindow *tmp)
 	 * and vice-versa.
 	 */
 
-	if (tmp->hints.flags & PBaseSize)
+	if (fw->hints.flags & PBaseSize)
 	{
-		if (tmp->hints.base_width < 0)
+		if (fw->hints.base_width < 0)
 		{
-			tmp->hints.base_width = 0;
+			fw->hints.base_width = 0;
 			if (!*broken_cause)
 			{
 				broken_cause = "base_width";
 			}
 		}
-		if (tmp->hints.base_height < 0)
+		if (fw->hints.base_height < 0)
 		{
-			tmp->hints.base_height = 0;
+			fw->hints.base_height = 0;
 			if (!*broken_cause)
 			{
 				broken_cause = "base_height";
 			}
 		}
-		if ((tmp->hints.base_width > tmp->hints.min_width) ||
-		    (tmp->hints.base_height > tmp->hints.min_height))
+		if ((fw->hints.base_width > fw->hints.min_width) ||
+		    (fw->hints.base_height > fw->hints.min_height))
 		{
 			/* In this case, doing the aspect ratio calculation
 			   for window_size - base_size as prescribed by the
@@ -2806,7 +2700,7 @@ void GetWindowSizeHints(FvwmWindow *tmp)
 			   in aspect ratio calculation while it is still used
 			   for grid sizing.
 			*/
-			tmp->hints.flags &= ~PBaseSize;
+			fw->hints.flags &= ~PBaseSize;
 #if 0
 			/* Keep silent about this, since the Xlib manual
 			 * actually recommends making min <= base <= max ! */
@@ -2816,37 +2710,37 @@ void GetWindowSizeHints(FvwmWindow *tmp)
 	}
 	else
 	{
-		if (tmp->hints.flags & PMinSize)
+		if (fw->hints.flags & PMinSize)
 		{
-			tmp->hints.base_width = tmp->hints.min_width;
-			tmp->hints.base_height = tmp->hints.min_height;
+			fw->hints.base_width = fw->hints.min_width;
+			fw->hints.base_height = fw->hints.min_height;
 		}
 		else
 		{
-			tmp->hints.base_width = 0;
-			tmp->hints.base_height = 0;
+			fw->hints.base_width = 0;
+			fw->hints.base_height = 0;
 		}
 	}
 
-	if (!(tmp->hints.flags & PWinGravity))
+	if (!(fw->hints.flags & PWinGravity))
 	{
-		tmp->hints.win_gravity = NorthWestGravity;
+		fw->hints.win_gravity = NorthWestGravity;
 	}
 
-	if ((tmp->hints.flags & PMaxSize) &&
-	    ((tmp->hints.flags & PMinSize) || (tmp->hints.flags & PBaseSize)))
+	if ((fw->hints.flags & PMaxSize) &&
+	    ((fw->hints.flags & PMinSize) || (fw->hints.flags & PBaseSize)))
 	{
-		if (tmp->hints.max_width < tmp->hints.base_width)
+		if (fw->hints.max_width < fw->hints.base_width)
 		{
-			tmp->hints.max_width = DEFAULT_MAX_MAX_WINDOW_WIDTH;
+			fw->hints.max_width = DEFAULT_MAX_MAX_WINDOW_WIDTH;
 			if (!*broken_cause)
 			{
 				broken_cause = "max_width";
 			}
 		}
-		if (tmp->hints.max_height < tmp->hints.base_height)
+		if (fw->hints.max_height < fw->hints.base_height)
 		{
-			tmp->hints.max_height = DEFAULT_MAX_MAX_WINDOW_HEIGHT;
+			fw->hints.max_height = DEFAULT_MAX_MAX_WINDOW_HEIGHT;
 			if (!*broken_cause)
 			{
 				broken_cause = "max_height";
@@ -2854,15 +2748,15 @@ void GetWindowSizeHints(FvwmWindow *tmp)
 		}
 	}
 
-	if (tmp->hints.flags & PAspect)
+	if (fw->hints.flags & PAspect)
 	{
 		/*
 		** check to make sure min/max aspect ratios look valid
 		*/
-#define maxAspectX tmp->hints.max_aspect.x
-#define maxAspectY tmp->hints.max_aspect.y
-#define minAspectX tmp->hints.min_aspect.x
-#define minAspectY tmp->hints.min_aspect.y
+#define maxAspectX fw->hints.max_aspect.x
+#define maxAspectY fw->hints.max_aspect.y
+#define minAspectX fw->hints.min_aspect.x
+#define minAspectY fw->hints.min_aspect.y
 
 
 		/*
@@ -2895,21 +2789,16 @@ void GetWindowSizeHints(FvwmWindow *tmp)
 			{
 				broken_cause = "aspect ratio";
 			}
-			tmp->hints.flags &= ~PAspect;
+			fw->hints.flags &= ~PAspect;
 			fvwm_msg(
 				WARN, "GetWindowSizeHints",
 				"The applicaton window (window id %#lx)\n"
 				"  \"%s\" has broken aspect ratio: "
 				"%d/%d - %d/%d\n"
-				"    fvwm is ignoring this aspect ratio.  "
-				"If you are having a problem with the\n"
-				"    application, send a bug report with this "
-				"message included to the\n"
-				"    application owner.  "
-				"There is no need to notify "
-				"fvwm-workers@fvwm.org.\n",
-				FW_W(tmp), tmp->name.name, minAspectX,
+				"    fvwm is ignoring this aspect ratio.  ",
+				FW_W(fw), fw->name.name, minAspectX,
 				minAspectY, maxAspectX, maxAspectY);
+			fvwm_msg_report_app();
 		}
 		else
 		{
@@ -2956,11 +2845,6 @@ void GetWindowSizeHints(FvwmWindow *tmp)
 			"The application window (id %#lx)\n"
 			"  \"%s\" has broken size hints (%s).\n"
 			"    fvwm is ignoring those hints.  "
-			"If you are having a problem with the\n"
-			"    application, send a bug report with this "
-			"message included to the\n"
-			"    application owner.  "
-			"There is no need to notify fvwm-workers@fvwm.org.\n"
 			"  hint override = %d, flags = %lx\n"
 			"  min_width = %d, min_height = %d, "
 			"max_width = %d, max_height = %d\n"
@@ -2968,8 +2852,8 @@ void GetWindowSizeHints(FvwmWindow *tmp)
 			"  min_aspect = %d/%d, max_aspect = %d/%d\n"
 			"  base_width = %d, base_height = %d\n"
 			"  win_gravity = %d\n",
-			FW_W(tmp), tmp->name.name, broken_cause,
-			HAS_OVERRIDE_SIZE_HINTS(tmp),
+			FW_W(fw), fw->name.name, broken_cause,
+			HAS_OVERRIDE_SIZE_HINTS(fw),
 			orig_hints.flags,
 			orig_hints.min_width, orig_hints.min_height,
 			orig_hints.max_width, orig_hints.max_height,
@@ -2978,6 +2862,7 @@ void GetWindowSizeHints(FvwmWindow *tmp)
 			orig_hints.max_aspect.x, orig_hints.max_aspect.y,
 			orig_hints.base_width, orig_hints.base_height,
 			orig_hints.win_gravity);
+		fvwm_msg_report_app();
 	}
 
 	return;
@@ -3136,8 +3021,8 @@ void destroy_window(FvwmWindow *fw)
 		}
 		adjust_fvwm_internal_windows(fw);
 		BroadcastPacket(
-			M_DESTROY_WINDOW, 3, FW_W(fw), FW_W_FRAME(fw),
-			(unsigned long)fw);
+			M_DESTROY_WINDOW, 3, (long)FW_W(fw),
+			(long)FW_W_FRAME(fw), (unsigned long)fw);
 		EWMH_DestroyWindow(fw);
 		focus_grab_buttons_on_layer(fw->layer);
 		Scr.FWScheduledForDestroy = flist_append_obj(
@@ -3154,8 +3039,9 @@ void destroy_window(FvwmWindow *fw)
 	{
 		SET_SCHEDULED_FOR_DESTROY(fw, 1);
 		adjust_fvwm_internal_windows(fw);
-		BroadcastPacket(M_DESTROY_WINDOW, 3,
-				FW_W(fw), FW_W_FRAME(fw), (unsigned long)fw);
+		BroadcastPacket(
+			M_DESTROY_WINDOW, 3, (long)FW_W(fw),
+			(long)FW_W_FRAME(fw), (unsigned long)fw);
 		EWMH_DestroyWindow(fw);
 	}
 	focus_grab_buttons_on_layer(fw->layer);
@@ -3255,13 +3141,11 @@ void RestoreWithdrawnLocation(
 		return;
 	}
 
-	if (HAS_NEW_WM_NORMAL_HINTS(fw))
-	{
-		/* get the latest size hints */
-		XSync(dpy, 0);
-		GetWindowSizeHints(fw);
-		SET_HAS_NEW_WM_NORMAL_HINTS(fw, 0);
-	}
+	/* always get the latest size hints in case the application changed
+	 * the gravity and we do not yet know about it */
+	XSync(dpy, 0);
+	GetWindowSizeHints(fw);
+	SET_HAS_NEW_WM_NORMAL_HINTS(fw, 0);
 	get_unshaded_geometry(fw, &unshaded_g);
 	gravity_get_naked_geometry(
 		fw->hints.win_gravity, fw, &naked_g, &unshaded_g);
@@ -3410,7 +3294,7 @@ void CaptureAllWindows(const exec_context_t *exc, Bool is_recapture)
 	memset(&win_opts, 0, sizeof(win_opts));
 	win_opts.flags.do_override_ppos = 1;
 	win_opts.flags.is_recapture = 1;
-	if (!(Scr.flags.windows_captured)) /* initial capture? */
+	if (!(Scr.flags.are_windows_captured)) /* initial capture? */
 	{
 		evh_args_t ea;
 		exec_context_changes_t ecc;
@@ -3466,7 +3350,7 @@ void CaptureAllWindows(const exec_context_t *exc, Bool is_recapture)
 				exc_destroy_context(ea.exc);
 			}
 		}
-		Scr.flags.windows_captured = 1;
+		Scr.flags.are_windows_captured = 1;
 	}
 	else /* must be recapture */
 	{
