@@ -1,3 +1,4 @@
+/* -*-c-*- */
 #ifndef FVWM_READ_H
 #define FVWM_READ_H
 
@@ -12,12 +13,12 @@ extern const char *get_current_read_dir(void);
 /**
  * Read and execute each line from stream.
  *
- * Parameters eventp, tmp_win, context, and Module supply the context
+ * Parameters eventp, fw, context, and Module supply the context
  * for executing the commands.
+ * cond_rc_t is passed thru incase piperead is running in a function.
  **/
-extern void run_command_stream( FILE* f,
-				XEvent *eventp, FvwmWindow *tmp_win,
-				unsigned long context, int Module );
+void run_command_stream(
+	cond_rc_t *cond_rc, FILE *f, const exec_context_t *exc);
 
 
 /**
@@ -27,8 +28,6 @@ extern void run_command_stream( FILE* f,
  * fvwm_userdir (set in main()) or in FVWM_DATADIR.  Return TRUE
  * if the file was found and executed.
  **/
-extern int run_command_file( char* filename,
-			     XEvent *eventp, FvwmWindow *tmp_win,
-			     unsigned long context, int Module );
+int run_command_file(char *filename, const exec_context_t *exc);
 
 #endif
