@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /*
  * FvwmButtons, copyright 1996, Jarl Totland
  *
@@ -31,8 +32,6 @@
 
 #include <ctype.h>
 #include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 
 #include <X11/Xlib.h>
 #include <X11/Intrinsic.h>
@@ -50,6 +49,7 @@ extern char *config_file;
 extern int button_width;
 extern int button_height;
 extern int has_button_geometry;
+extern int screen;
 
 /* contains the character that terminated the last string from seekright */
 static char terminator = '\0';
@@ -114,6 +114,7 @@ static char *my_get_font(char **s)
 /**
 *** ParseBack()
 *** Parses the options possible to Back
+*** This function appears to be obsolete & should be removed -- SS.
 **/
 static int ParseBack(char **ss)
 {
@@ -154,7 +155,7 @@ static int ParseBack(char **ss)
 *** ParseBoxSize()
 *** Parses the options possible to BoxSize
 **/
-static void ParseBoxSize(char **ss, unsigned long *flags)
+static void ParseBoxSize(char **ss, flags_type *flags)
 {
   char *opts[] =
   {
@@ -172,18 +173,20 @@ static void ParseBoxSize(char **ss, unsigned long *flags)
   switch(m)
   {
   case 0:
-    *flags &= ~(b_SizeFixed|b_SizeSmart);
+    flags->b_SizeFixed = 0;
+	flags->b_SizeSmart = 0;
     break;
   case 1:
-    *flags |= b_SizeFixed;
-    *flags &= ~b_SizeSmart;
+    flags->b_SizeFixed = 1;
+    flags->b_SizeSmart = 0;
     break;
   case 2:
-    *flags |= b_SizeSmart;
-    *flags &= ~b_SizeFixed;
+    flags->b_SizeSmart = 1;
+    flags->b_SizeFixed = 0;
     break;
   default:
-    *flags &= ~(b_SizeFixed|b_SizeSmart);
+    flags->b_SizeFixed = 0;
+	flags->b_SizeSmart = 0;
     fprintf(stderr,"%s: Illegal boxsize option \"%s\"\n",MyName, s);
     break;
   }
@@ -215,20 +218,24 @@ static void ParseTitle(char **ss, byte *flags, byte *mask)
     {
     case 0: /* Left */
       *flags&=~b_TitleHoriz;
+      *flags&=~3;
       *mask|=b_TitleHoriz;
       break;
     case 1: /* Right */
       *flags&=~b_TitleHoriz;
+      *flags&=~3;
       *flags|=2;
       *mask|=b_TitleHoriz;
       break;
     case 2: /* Center */
       *flags&=~b_TitleHoriz;
+      *flags&=~3;
       *flags|=1;
       *mask|=b_TitleHoriz;
       break;
     case 3: /* Side */
       *flags|=b_Horizontal;
+      *flags&=~3;
       *mask|=b_Horizontal;
       break;
     default:
@@ -246,7 +253,8 @@ static void ParseTitle(char **ss, byte *flags, byte *mask)
 *** ParseSwallow()
 *** Parses the options possible to Swallow
 **/
-static void ParseSwallow(char **ss,byte *flags,byte *mask)
+static void ParseSwallow(
+	char **ss,unsigned int *flags,unsigned int *mask, button_info *b)
 {
   char *swallowopts[] =
   {
@@ -256,6 +264,8 @@ static void ParseSwallow(char **ss,byte *flags,byte *mask)
     "respawn","norespawn",
     "useold", "noold",
     "usetitle", "notitle",
+    "fvwmmodule", "nofvwmmodule",
+    "swallownew",
     NULL
   };
   char *t,*s=*ss;
@@ -294,10 +304,12 @@ static void ParseSwallow(char **ss,byte *flags,byte *mask)
     case 6: /* Respawn */
       *flags|=b_Respawn;
       *mask|=b_Respawn;
+      b->newflags.do_swallow_new = 0;
       break;
     case 7: /* NoRespawn */
       *flags&=~b_Respawn;
       *mask|=b_Respawn;
+      b->newflags.do_swallow_new = 0;
       break;
     case 8: /* UseOld */
       *flags|=b_UseOld;
@@ -315,6 +327,19 @@ static void ParseSwallow(char **ss,byte *flags,byte *mask)
       *flags&=~b_UseTitle;
       *mask|=b_UseTitle;
       break;
+    case 12: /* FvwmModule */
+      *flags|=b_FvwmModule;
+      *mask|=b_FvwmModule;
+      break;
+    case 13: /* NoFvwmModule */
+      *flags&=~b_FvwmModule;
+      *mask|=b_FvwmModule;
+      break;
+    case 14: /* SwallowNew */
+      *flags&=~b_Respawn;
+      *mask|=b_Respawn;
+      b->newflags.do_swallow_new = 1;
+      break;
     default:
       t=seekright(&s);
       fprintf(stderr,"%s: Illegal Swallow option \"%s\"\n",MyName,
@@ -331,8 +356,9 @@ static void ParseSwallow(char **ss,byte *flags,byte *mask)
 *** ParsePanel()
 *** Parses the options possible to Panel
 **/
-static void ParsePanel(char **ss, byte *flags, byte *mask, char *direction,
-		       int *steps, int *delay, panel_flags_type *panel_flags,
+static void ParsePanel(char **ss, unsigned int *flags, unsigned int *mask,
+		       char *direction, int *steps, int *delay,
+		       panel_flags_type *panel_flags,
 		       int *indicator_size, int *rela_x, int *rela_y,
 		       char *position, char *context)
 {
@@ -463,7 +489,7 @@ static void ParsePanel(char **ss, byte *flags, byte *mask, char *direction,
       {
 	s = trimleft(s);
 	/* get x and y offset */
-	if((*s>='0' && *s<='9') || *s=='+' || *s=='-')
+	if(isdigit(*s) || *s=='+' || *s=='-')
 	{
 	  sscanf(s, "%i%n", rela_x, &n);
 	  s += n;
@@ -588,42 +614,36 @@ static void ParseContainer(char **ss,button_info *b)
       break;
     case 2: /* Font */
       if (b->c->font_string)
-        free(b->c->font_string);
-      b->font_string = my_get_font(&s);
-      if(b->c->font_string)
-      {
-	b->c->flags|=b_Font;
-      } else
-	b->c->flags&=~b_Font;
+	free(b->c->font_string);
+      b->c->font_string = my_get_font(&s);
+      b->c->flags.b_Font = (b->c->font_string ? 1 : 0);
       break;
     case 3: /* Frame */
       b->c->framew=strtol(s,&t,10);
-      b->c->flags|=b_Frame;
+      b->c->flags.b_Frame = 1;
       s=t;
       break;
     case 4: /* Back */
       s = trimleft(s);
       if(*s=='(' && s++)
 	if(ParseBack(&s))
-	  b->c->flags|=b_IconBack;
+	  b->c->flags.b_IconBack = 1;
       if (b->c->back) free(b->c->back);
       b->c->back=seekright(&s);
       if(b->c->back)
       {
-	b->c->flags|=b_Back;
+	b->c->flags.b_Back = 1;
       }
       else
-	b->c->flags&=~(b_IconBack|b_Back);
+      {
+	b->c->flags.b_IconBack = 0;
+	b->c->flags.b_Back = 0;
+      }
       break;
     case 5: /* Fore */
       if (b->c->fore) free(b->c->fore);
       b->c->fore=seekright(&s);
-      if(b->c->fore)
-      {
-	b->c->flags|=b_Fore;
-      }
-      else
-	b->c->flags&=~b_Fore;
+      b->c->flags.b_Fore = (b->c->fore ? 1 : 0);
       break;
     case 6: /* Padding */
       i=strtol(s,&t,10);
@@ -637,7 +657,7 @@ static void ParseContainer(char **ss,button_info *b)
 	  b->c->ypad=i;
 	  s=t;
 	}
-	b->c->flags|=b_Padding;
+	b->c->flags.b_Padding = 1;
       }
       else
 	fprintf(stderr,"%s: Illegal padding argument\n",MyName);
@@ -650,7 +670,7 @@ static void ParseContainer(char **ss,button_info *b)
 	b->c->justify_mask=0;
 	ParseTitle(&s,&b->c->justify,&b->c->justify_mask);
 	if(b->c->justify_mask)
-	  b->c->flags|=b_Justify;
+	  b->c->flags.b_Justify = 1;
       }
       else
       {
@@ -667,20 +687,20 @@ static void ParseContainer(char **ss,button_info *b)
 	Bool failed = False;
 
 	s = trimleft(s);
-	if (b->c->flags & (b_Swallow | b_Panel))
+	if (b->c->flags.b_Swallow || b->c->flags.b_Panel)
 	{
 	  fprintf(stderr, "%s: Multiple Swallow or Panel options are not"
-		" allowed in a signle button", MyName);
+		" allowed in a single button", MyName);
 	  failed = True;
 	}
 	else if(*s=='(' && s++)
 	{
 	  b->c->swallow=0;
 	  b->c->swallow_mask=0;
-	  ParseSwallow(&s, &b->c->swallow, &b->c->swallow_mask);
+	  ParseSwallow(&s, &b->c->swallow, &b->c->swallow_mask, b);
 	  if(b->c->swallow_mask)
 	  {
-	    b->c->flags |= b_Swallow;
+	    b->c->flags.b_Swallow = 1;
 	  }
 	}
 	else
@@ -701,7 +721,7 @@ static void ParseContainer(char **ss,button_info *b)
       }
       break;
     case 9: /* NoSize */
-      b->c->flags|=b_Size;
+      b->c->flags.b_Size = 1;
       b->c->minx=b->c->miny=0;
       break;
 
@@ -712,7 +732,7 @@ static void ParseContainer(char **ss,button_info *b)
       {
 	b->c->minx=i;
 	b->c->miny=j;
-	b->c->flags|=b_Size;
+	b->c->flags.b_Size = 1;
 	s=o;
       }
       else
@@ -728,20 +748,19 @@ static void ParseContainer(char **ss,button_info *b)
       if(t > s)
       {
 	b->c->colorset = i;
-	b->c->flags |= b_Colorset;
+	b->c->flags.b_Colorset = 1;
 	AllocColorset(i);
 	s = t;
       }
       else
       {
-	b->c->flags &= ~b_Colorset;
+	b->c->flags.b_Colorset = 0;
       }
       break;
 
     default:
       t=seekright(&s);
-      fprintf(stderr,"%s: Illegal container option \"%s\"\n",MyName,
-	      (t)?t:"");
+      fprintf(stderr,"%s: Illegal container option \"%s\"\n",MyName, (t)?t:"");
       if (t)
 	free(t);
     }
@@ -755,7 +774,7 @@ static void ParseContainer(char **ss,button_info *b)
 *** ParseButton()
 ***
 *** parse a buttonconfig string.
-*** *FvwmButtons(option[ options]) title iconname command
+*** *FvwmButtons: (option[ options]) title iconname command
 **/
 /*#define DEBUG_PARSER*/
 static void ParseButton(button_info **uberb,char *s)
@@ -781,6 +800,7 @@ static void ParseButton(button_info **uberb,char *s)
       "swallow",
       "panel",
       "actionignoresclientwindow",
+      "actiononpress",
       "container",
       "end",
       "nosize",
@@ -790,6 +810,13 @@ static void ParseButton(button_info **uberb,char *s)
       "center",
       "colorset",
       "action",
+      "id",
+      "activeicon",
+      "activetitle",
+      "pressicon",
+      "presstitle",
+      "activecolorset",
+      "presscolorset",
       NULL
     };
     s = trimleft(s);
@@ -797,7 +824,13 @@ static void ParseButton(button_info **uberb,char *s)
     {
       Bool is_swallow = False;
 
-      if((*s>='0' && *s<='9') || *s=='+' || *s=='-')
+      if (*s == ',')
+      {
+	      s++;
+	      s = trimleft(s);
+	      continue;
+      }
+      if(isdigit(*s) || *s=='+' || *s=='-')
       {
 	char *geom;
 	int x,y,flags;
@@ -813,12 +846,12 @@ static void ParseButton(button_info **uberb,char *s)
 	  if(flags&XValue)
 	  {
 	    b->BPosX=x;
-	    b->flags|=b_PosFixed;
+	    b->flags.b_PosFixed = 1;
 	  }
 	  if(flags&YValue)
 	  {
 	    b->BPosY=y;
-	    b->flags|=b_PosFixed;
+	    b->flags.b_PosFixed = 1;
 	  }
 	  if(flags&XNegative)
 	    b->BPosX=-1-x;
@@ -829,48 +862,39 @@ static void ParseButton(button_info **uberb,char *s)
 	s = trimleft(s);
 	continue;
       }
-      if(*s==',' && s++)
-	s = trimleft(s);
       switch(GetTokenIndex(s,opts,-1,&s))
       {
       case 0: /* Back */
 	s = trimleft(s);
 	if(*s=='(' && s++)
 	  if(ParseBack(&s))
-	    b->flags|=b_IconBack;
-	if(b->flags&b_Back && b->back)
+	    b->flags.b_IconBack = 1;
+	if(b->flags.b_Back && b->back)
 	  free(b->back);
 	b->back=seekright(&s);
 	if(b->back)
 	{
-	  b->flags|=b_Back;
+	  b->flags.b_Back = 1;
 	}
 	else
-	  b->flags&=~(b_IconBack|b_Back);
+	{
+	  b->flags.b_IconBack = 0;
+	  b->flags.b_Back = 0;
+	}
 	break;
 
       case 1: /* Fore */
-	if(b->flags&b_Fore && b->fore)
+	if(b->flags.b_Fore && b->fore)
 	  free(b->fore);
 	b->fore=seekright(&s);
-	if(b->fore)
-	{
-	  b->flags|=b_Fore;
-	}
-	else
-	  b->flags&=~b_Fore;
+	b->flags.b_Fore = (b->fore ? 1 : 0);
 	break;
 
       case 2: /* Font */
-	if(b->flags&b_Font && b->font_string)
+	if(b->flags.b_Font && b->font_string)
 	  free(b->font_string);
-        b->font_string = my_get_font(&s);
-	if(b->font_string)
-	{
-	  b->flags|=b_Font;
-	}
-	else
-	  b->flags&=~b_Font;
+	b->font_string = my_get_font(&s);
+	b->flags.b_Font = (b->font_string ? 1 : 0);
 	break;
 
 	/* --------------------------- Title ------------------------- */
@@ -883,7 +907,7 @@ static void ParseButton(button_info **uberb,char *s)
 	  b->justify_mask=0;
 	  ParseTitle(&s,&b->justify,&b->justify_mask);
 	  if(b->justify_mask)
-	    b->flags|=b_Justify;
+	    b->flags.b_Justify = 1;
 	}
 	t=seekright(&s);
 	if(t && *t && (t[0]!='-' || t[1]!=0))
@@ -894,7 +918,7 @@ static void ParseButton(button_info **uberb,char *s)
 #ifdef DEBUG_PARSER
 	  fprintf(stderr,"PARSE: Title \"%s\"\n",b->title);
 #endif
-	  b->flags|=b_Title;
+	  b->flags.b_Title = 1;
 	}
 	else
 	{
@@ -909,7 +933,7 @@ static void ParseButton(button_info **uberb,char *s)
 	t=seekright(&s);
 	if(t && *t && (t[0] != '-' || t[1] != 0))
 	{
-	  if (b->flags & b_Swallow)
+	  if (b->flags.b_Swallow)
 	  {
 	    fprintf(
 	      stderr,"%s: a button can not have an icon and a swallowed window"
@@ -920,13 +944,12 @@ static void ParseButton(button_info **uberb,char *s)
 	    if (b->icon_file)
 	      free(b->icon_file);
 	    b->icon_file=t;
-	    b->IconWin=None;
-	    b->flags|=b_Icon;
+	    b->flags.b_Icon = 1;
 	  }
 	}
 	else
 	{
-	  fprintf(stderr,"%s: Missing icon argument\n",MyName);
+	  fprintf(stderr,"%s: Missing Icon argument\n",MyName);
 	  if(t)free(t);
 	}
 	break;
@@ -937,7 +960,7 @@ static void ParseButton(button_info **uberb,char *s)
 	i=strtol(s,&t,10);
 	if(t>s)
 	{
-	  b->flags|=b_Frame;
+	  b->flags.b_Frame = 1;
 	  b->framew=i;
 	  s=t;
 	}
@@ -952,7 +975,7 @@ static void ParseButton(button_info **uberb,char *s)
 	if(t>s)
 	{
 	  b->xpad=b->ypad=i;
-	  b->flags |= b_Padding;
+	  b->flags.b_Padding = 1;
 	  s=t;
 	  i=strtol(s,&t,10);
 	  if(t>s)
@@ -993,7 +1016,7 @@ static void ParseButton(button_info **uberb,char *s)
 	if(*s=='(' && s++)
 	{
 	  if (is_swallow)
-	    ParseSwallow(&s, &b->swallow,&b->swallow_mask);
+	    ParseSwallow(&s, &b->swallow,&b->swallow_mask, b);
 	  else
 	    ParsePanel(&s, &b->swallow, &b->swallow_mask, &b->slide_direction,
 		       &b->slide_steps, &b->slide_delay_ms, &b->panel_flags,
@@ -1009,19 +1032,21 @@ static void ParseButton(button_info **uberb,char *s)
 	  b->hangon=t;
 	  if (is_swallow)
 	  {
-	    if (b->flags & b_Icon)
+	    if (b->flags.b_Icon)
 	    {
 	      fprintf(
 		stderr,"%s: a button can not have an icon and a swallowed "
 		" window at the same time. Ignoring icon", MyName);
-	      b->flags &= ~ b_Icon;
+	      b->flags.b_Icon = 0;
 	    }
 
-	    b->flags |= (b_Swallow | b_Hangon);
+	    b->flags.b_Swallow = 1;
+	    b->flags.b_Hangon = 1;
 	  }
 	  else
 	  {
-	    b->flags |= (b_Panel | b_Hangon);
+	    b->flags.b_Panel = 1;
+	    b->flags.b_Hangon = 1;
 	    b->newflags.is_panel = 1;
 	    b->newflags.panel_mapped = 0;
 	  }
@@ -1032,11 +1057,13 @@ static void ParseButton(button_info **uberb,char *s)
 	  {
 	    char *p;
 
-	    p = expand_action(o, NULL);
+	    p = module_expand_action(
+		    Dpy, screen, o, NULL, UberButton->c->fore,
+		    UberButton->c->back);
 	    if (p)
 	    {
 	      if(!(buttonSwallow(b)&b_UseOld))
-		SendText(fd,p,0);
+		exec_swallow(p,b);
 	      if (b->spawn)
 		free(b->spawn);
 	      b->spawn=o;  /* Might be needed if respawning sometime */
@@ -1052,16 +1079,38 @@ static void ParseButton(button_info **uberb,char *s)
 	  if(o)
 	    free(o);
 	}
+	/* check if it is a module by command line inspection if
+	 * this hints has not been given in the swallow option */
+	if (is_swallow && !(b->swallow_mask & b_FvwmModule))
+	{
+	  if (b->spawn != NULL &&
+	      (strncasecmp(b->spawn,"module",6) == 0 ||
+	       strncasecmp(b->spawn,"fvwm",4) == 0))
+	  {
+	    b->swallow |= b_FvwmModule;
+	  }
+	}
 	break;
 
       case 9: /* ActionIgnoresClientWindow */
-	b->flags |= b_ActionIgnoresClientWindow;
+	b->flags.b_ActionIgnoresClientWindow = 1;
+	break;
+
+      case 10: /* ActionIgnoresClientWindow */
+	b->flags.b_ActionOnPress = 1;
 	break;
 
 	/* -------------------------- container ---------------------- */
 
-      case 10: /* Container */
-	b->flags&=b_Frame|b_Back|b_Fore|b_Padding|b_Action;
+      case 11: /* Container */
+	/*
+	 * SS: This looks very buggy! The FvwmButtons(1) man page suggests
+	 * it's here to restrict the options used with "Container", but it
+	 * only restricts those options appearing _before_ the "Container"
+	 * keyword for a button.
+	 * b->flags&=b_Frame|b_Back|b_Fore|b_Padding|b_Action;
+	 */
+
 	MakeContainer(b);
 	*uberb=b;
 	s = trimleft(s);
@@ -1069,7 +1118,7 @@ static void ParseButton(button_info **uberb,char *s)
 	  ParseContainer(&s,b);
 	break;
 
-      case 11: /* End */
+      case 12: /* End */
 	*uberb=ub->parent;
 	ub->c->buttons[--(ub->c->num_buttons)]=NULL;
 	free(b);
@@ -1078,59 +1127,69 @@ static void ParseButton(button_info **uberb,char *s)
 	  fprintf(stderr,"%s: Unmatched END in config file\n",MyName);
 	  exit(1);
 	}
+	if (ub->parent->c->flags.b_Colorset ||
+	    ub->parent->c->flags.b_ColorsetParent)
+	{
+		ub->c->flags.b_ColorsetParent = 1;
+	}
+	if (ub->parent->c->flags.b_IconBack || ub->parent->c->flags.b_IconParent)
+	{
+		ub->c->flags.b_IconParent = 1;
+	}
 	return;
 
-      case 12: /* NoSize */
-	b->flags|=b_Size;
+      case 13: /* NoSize */
+	b->flags.b_Size = 1;
 	b->minx=b->miny=0;
 	break;
 
-      case 13: /* Size */
+      case 14: /* Size */
 	i=strtol(s,&t,10);
 	j=strtol(t,&o,10);
 	if(t>s && o>t)
 	{
 	  b->minx=i;
 	  b->miny=j;
-	  b->flags|=b_Size;
+	  b->flags.b_Size = 1;
 	  s=o;
 	}
 	else
 	  fprintf(stderr,"%s: Illegal size arguments\n",MyName);
 	break;
 
-      case 14: /* Left */
-	b->flags |= b_Left;
-	b->flags &= ~b_Right;
+      case 15: /* Left */
+	b->flags.b_Left = 1;
+	b->flags.b_Right = 0;
 	break;
 
-      case 15: /* Right */
-	b->flags |= b_Right;
-	b->flags &= ~b_Left;
+      case 16: /* Right */
+	b->flags.b_Right = 1;
+	b->flags.b_Left = 0;
 	break;
 
-      case 16: /* Center */
-	b->flags &= ~(b_Right|b_Left);
+      case 17: /* Center */
+	b->flags.b_Right = 0;
+	b->flags.b_Left = 0;
 	break;
 
-      case 17: /* Colorset */
+      case 18: /* Colorset */
 	i = strtol(s, &t, 10);
 	if(t > s)
 	{
 	  b->colorset = i;
-	  b->flags |= b_Colorset;
+	  b->flags.b_Colorset = 1;
 	  s=t;
 	  AllocColorset(i);
 	}
 	else
 	{
-	  b->flags &= ~b_Colorset;
+	  b->flags.b_Colorset = 0;
 	}
 	break;
 
 	/* --------------------------- action ------------------------ */
 
-      case 18: /* Action */
+      case 19: /* Action */
 	s = trimleft(s);
 	i=0;
 	if(*s=='(')
@@ -1146,27 +1205,27 @@ static void ParseButton(button_info **uberb,char *s)
 	  while(*s && *s!=')')
 	    s++;
 	  if(*s==')')
-            s++;
+	    s++;
 	}
-        {
-          char *r;
-          char *u = s;
+	{
+	  char *r;
+	  char *u = s;
 
-          s = GetQuotedString(s, &t, ",)", NULL, "(", ")");
-          r = s;
-          if (t && r > u + 1)
-          {
-            /* remove unquoted trailing spaces */
-            r -= 2;
-            while (r >= u && isspace(*r))
-              r--;
-            r++;
-            if (isspace(*r))
-            {
-              t[strlen(t) - (s - r - 1)] = 0;
-            }
-          }
-        }
+	  s = GetQuotedString(s, &t, ",)", NULL, "(", ")");
+	  r = s;
+	  if (t && r > u + 1)
+	  {
+	    /* remove unquoted trailing spaces */
+	    r -= 2;
+	    while (r >= u && isspace(*r))
+	      r--;
+	    r++;
+	    if (isspace(*r))
+	    {
+	      t[strlen(t) - (s - r - 1)] = 0;
+	    }
+	  }
+	}
 	if(t)
 	{
 	  AddButtonAction(b,i,t);
@@ -1176,6 +1235,181 @@ static void ParseButton(button_info **uberb,char *s)
 	  fprintf(stderr,"%s: Missing action argument\n",MyName);
 	break;
 
+      case 20: /* Id */
+	s = trimleft(s);
+	s = DoGetNextToken(s, &t, NULL, ",)", &terminator);
+
+	/* it should include the delimiter */
+	if (s && terminator == ')')
+	{
+	  s--;
+	}
+
+	if (t)
+	{
+	  if (isalpha(t[0]))
+	  {
+	    /* should check for duplicate ids first... */
+	    b->flags.b_Id = 1;
+	    if (b->id)
+	      free(b->id);
+	    CopyString(&b->id, t);
+	  }
+          else
+	  {
+	    fprintf(stderr, "%s: Incorrect id '%s' ignored\n", MyName, t);
+	  }
+	  free(t);
+	}
+	else
+	{
+	  fprintf(stderr, "%s: Missing id argument\n", MyName);
+	}
+	break;
+
+	/* ---------------------------- ActiveIcon ------------------------- */
+	case 21: /* ActiveIcon */
+		t = seekright(&s);
+		if (t && *t && (t[0] != '-' || t[1] != 0))
+		{
+			if (b->flags.b_Swallow)
+			{
+				fprintf(stderr,"%s: a button can not have a "
+					"ActiveIcon and a swallowed window at "
+					"the same time. Ignoring ActiveIcon.",
+					MyName);
+			}
+			else
+			{
+				if (b->active_icon_file)
+					free(b->active_icon_file);
+				b->active_icon_file = t;
+				b->flags.b_ActiveIcon = 1;
+			}
+		}
+		else
+		{
+			fprintf(stderr,"%s: Missing ActiveIcon argument\n",
+				MyName);
+			if (t)
+				free(t);
+		}
+		break;
+
+	/* ------------------------- ActiveTitle ------------------------- */
+	case 22: /* ActiveTitle */
+		s = trimleft(s);
+		if (*s=='(')
+		{
+			fprintf(stderr,"%s: justification not allowed for "
+				"ActiveTitle.\n", MyName);
+		}
+		t = seekright(&s);
+		if(t && *t && (t[0] != '-' || t[1] != 0))
+		{
+			if (b->activeTitle)
+				free(b->activeTitle);
+			b->activeTitle = t;
+#ifdef DEBUG_PARSER
+			fprintf(stderr,"PARSE: ActiveTitle \"%s\"\n",
+				b->activeTitle);
+#endif
+			b->flags.b_ActiveTitle = 1;
+		}
+		else
+		{
+			fprintf(stderr,"%s: Missing ActiveTitle argument\n",
+					MyName);
+			if (t)
+				free(t);
+		}
+		break;
+
+	/* ------------------------- PressIcon ------------------------- */
+	case 23: /* PressIcon */
+		t = seekright(&s);
+		if (t && *t && (t[0] != '-' || t[1] != 0))
+		{
+			if (b->flags.b_Swallow)
+			{
+				fprintf(stderr,"%s: a button can not have a "
+					"PressIcon and a swallowed window at "
+					"the same time. Ignoring PressIcon.",
+					MyName);
+			}
+			else
+			{
+				if (b->press_icon_file)
+					free(b->press_icon_file);
+				b->press_icon_file = t;
+				b->flags.b_PressIcon = 1;
+			}
+		}
+		else
+		{
+			fprintf(stderr,"%s: Missing PressIcon argument\n",
+				MyName);
+			if (t)
+				free(t);
+		}
+		break;
+
+	/* ------------------------- PressTitle ------------------------- */
+	case 24: /* PressTitle */
+		s = trimleft(s);
+		if (*s=='(')
+		{
+			fprintf(stderr,"%s: justification not allowed for "
+				"PressTitle.\n", MyName);
+		}
+		t = seekright(&s);
+		if (t && *t && (t[0] != '-' || t[1] != 0))
+		{
+			if (b->pressTitle)
+				free(b->pressTitle);
+			b->pressTitle = t;
+#ifdef DEBUG_PARSER
+			fprintf(stderr,"PARSE: PressTitle \"%s\"\n",
+				b->pressTitle);
+#endif
+			b->flags.b_PressTitle = 1;
+		}
+		else
+		{
+			fprintf(stderr,"%s: Missing PressTitle argument\n",
+				MyName);
+			if (t)
+				free(t);
+		}
+		break;
+
+	/* ------------------------- ------------------------- */
+	case 25: /* ActiveColorset */
+		i = strtol(s, &t, 10);
+		if (t > s)
+		{
+			b->activeColorset = i;
+			b->flags.b_ActiveColorset = 1;
+			s = t;
+			AllocColorset(i);
+		}
+		else
+			b->flags.b_ActiveColorset = 0;
+		break;
+	/* ------------------------- ------------------------- */
+	case 26: /* PressColorset */
+		i = strtol(s, &t, 10);
+		if (t > s)
+		{
+			b->pressColorset = i;
+			b->flags.b_PressColorset = 1;
+			s = t;
+			AllocColorset(i);
+		}
+		else
+			b->flags.b_PressColorset = 0;
+		break;
+	/* ------------------------- ------------------------- */
       default:
 	t=seekright(&s);
 	fprintf(stderr,"%s: Illegal button option \"%s\"\n",MyName,
@@ -1183,7 +1417,7 @@ static void ParseButton(button_info **uberb,char *s)
 	if (t)
 	  free(t);
 	break;
-      }
+      } /* end switch */
       s = trimleft(s);
     }
     if (s && *s)
@@ -1194,11 +1428,11 @@ static void ParseButton(button_info **uberb,char *s)
   }
 
   /* get title and iconname */
-  if(!(b->flags&b_Title))
+  if(!b->flags.b_Title)
   {
     b->title=seekright(&s);
     if(b->title && *b->title && ((b->title)[0]!='-'||(b->title)[1]!=0))
-      b->flags |= b_Title;
+      b->flags.b_Title = 1;
     else
       if(b->title)free(b->title);
   }
@@ -1210,14 +1444,13 @@ static void ParseButton(button_info **uberb,char *s)
       free(temp);
   }
 
-  if(!(b->flags&b_Icon))
+  if(!b->flags.b_Icon)
   {
     b->icon_file=seekright(&s);
     if(b->icon_file && b->icon_file &&
        ((b->icon_file)[0]!='-'||(b->icon_file)[1]!=0))
     {
-      b->flags|=b_Icon;
-      b->IconWin=None;
+      b->flags.b_Icon = 1;
     }
     else
       if(b->icon_file)free(b->icon_file);
@@ -1235,7 +1468,7 @@ static void ParseButton(button_info **uberb,char *s)
   /* Swallow hangon command */
   if (strncasecmp(s,"swallow",7)==0 || strncasecmp(s,"panel",7)==0)
   {
-    if(b->flags & (b_Swallow | b_Panel))
+    if(b->flags.b_Swallow || b->flags.b_Panel)
     {
       fprintf(stderr,"%s: Illegal with both old and new swallow!\n",
 	      MyName);
@@ -1255,9 +1488,15 @@ static void ParseButton(button_info **uberb,char *s)
     if (!b->hangon)
       b->hangon = safestrdup("");
     if (tolower(*s) == 's')
-      b->flags |= b_Swallow | b_Hangon;
+    {
+      b->flags.b_Swallow = 1;
+      b->flags.b_Hangon = 1;
+    }
     else
-      b->flags |= b_Panel | b_Hangon;
+    {
+      b->flags.b_Panel = 1;
+      b->flags.b_Hangon = 1;
+    }
     b->swallow|=1;
     s = trimleft(s);
     if(!(b->swallow&b_NoHints))
@@ -1295,6 +1534,8 @@ static void ParseConfigLine(button_info **ubb,char *s)
     "pixmap",
     "boxsize",
     "colorset",
+    "activecolorset",
+    "presscolorset",
     NULL
   };
   int i,j,k;
@@ -1308,7 +1549,7 @@ static void ParseConfigLine(button_info **ubb,char *s)
     i=sscanf(s,"%63s",geom);
     if(i==1)
     {
-      parse_window_geometry(geom);
+      parse_window_geometry(geom, 0);
     }
     break;
   }
@@ -1319,39 +1560,14 @@ static void ParseConfigLine(button_info **ubb,char *s)
     i=sscanf(s,"%63s",geom);
     if(i==1)
     {
-      int flags;
-      int g_x;
-      int g_y;
-      unsigned int width;
-      unsigned int height;
-
-      flags = XParseGeometry(geom,&g_x,&g_y,&width,&height);
-      if (!flags)
-	break;
-      UberButton->w = 0;
-      UberButton->h = 0;
-      UberButton->x = 0;
-      UberButton->y = 0;
-      if (flags&WidthValue)
-	button_width = width;
-      if (flags&HeightValue)
-	button_height = height;
-      if (flags&XValue)
-	UberButton->x = g_x;
-      if (flags&YValue)
-	UberButton->y = g_y;
-      if (flags&XNegative)
-	UberButton->w = 1;
-      if (flags&YNegative)
-	UberButton->h = 1;
-      has_button_geometry = 1;
+      parse_window_geometry(geom, 1);
     }
     break;
   }
   case 2:/* Font */
     if (ub->c->font_string)
       free(ub->c->font_string);
-    CopyString(&ub->c->font_string, s);
+    CopyStringWithQuotes(&ub->c->font_string, s);
     break;
   case 3:/* Padding */
     i=sscanf(s,"%d %d",&j,&k);
@@ -1393,14 +1609,14 @@ static void ParseConfigLine(button_info **ubb,char *s)
   case 10:/* Pixmap */
     s = trimleft(s);
     if (strncasecmp(s,"none",4)==0)
-      ub->c->flags|=b_TransBack;
+      ub->c->flags.b_TransBack = 1;
     else
     {
       if (ub->c->back_file)
 	free(ub->c->back_file);
       CopyString(&(ub->c->back_file),s);
     }
-    ub->c->flags|=b_IconBack;
+    ub->c->flags.b_IconBack = 1;
     break;
   case 11: /* BoxSize */
     ParseBoxSize(&s, &ub->c->flags);
@@ -1410,14 +1626,42 @@ static void ParseConfigLine(button_info **ubb,char *s)
     if (i > 0)
     {
       ub->c->colorset = j;
-      ub->c->flags |= b_Colorset;
+      ub->c->flags.b_Colorset = 1;
       AllocColorset(j);
     }
     else
     {
-      ub->c->flags &= ~b_Colorset;
+      ub->c->flags.b_Colorset = 0;
     }
     break;
+  case 13: /* ActiveColorset */
+    i = sscanf(s, "%d", &j);
+    if (i > 0)
+    {
+      ub->c->activeColorset = j;
+      ub->c->flags.b_ActiveColorset = 1;
+      AllocColorset(j);
+    }
+    else
+    {
+      ub->c->flags.b_ActiveColorset = 0;
+    }
+    break;
+
+  case 14: /* PressColorset */
+    i = sscanf(s, "%d", &j);
+    if (i > 0)
+    {
+      ub->c->pressColorset = j;
+      ub->c->flags.b_PressColorset = 1;
+      AllocColorset(j);
+    }
+    else
+    {
+      ub->c->flags.b_PressColorset = 0;
+    }
+    break;
+
   default:
     s = trimleft(s);
     ParseButton(ubb,s);
@@ -1467,172 +1711,7 @@ static void ParseConfigFile(button_info *ub)
   fclose(f);
 }
 
-char *expand_action(char *in_action, button_info *b)
-{
-  char *variables[] =
-  {
-    "$",
-    "fg",
-    "bg",
-    "left",
-    "-left",
-    "right",
-    "-right",
-    "top",
-    "-top",
-    "bottom",
-    "-bottom",
-    "width",
-    "height",
-    NULL
-  };
-  char *action = NULL;
-  char *src;
-  char *dest;
-  char *string = NULL;
-  char *rest;
-  int px;
-  int py;
-  int val = 0;
-  int offset;
-  int x;
-  int y;
-  int f;
-  int i;
-  unsigned int w = 0;
-  unsigned int h = 0;
-  Window win;
-  extern int dpw;
-  extern int dph;
-
-  /* create a temporary storage for expanding */
-  action = (char *)malloc(MAX_MODULE_INPUT_TEXT_LEN);
-  if (!action)
-  {
-    /* could not alloc memory */
-    return NULL;
-  }
-
-  /* calculate geometry */
-  if (b)
-  {
-    w = buttonWidth(b);
-    h = buttonHeight(b);
-    buttonInfo(b, &x, &y, &px, &py, &f);
-    XTranslateCoordinates(Dpy, MyWindow, Root, x, y, &x, &y, &win);
-  }
-
-  for (src = in_action, dest = action; *src != 0; src++)
-  {
-    if (*src != '$')
-    {
-      *(dest++) = *src;
-    }
-    else
-    {
-      char *dest_org = dest;
-      Bool is_string = False;
-      Bool is_value = False;
-
-      *(dest++) = *(src++);
-      i = GetTokenIndex(src, variables, -1, &rest);
-      if (i == -1)
-      {
-	src--;
-	continue;
-      }
-      switch (i)
-      {
-      case 0: /* $ */
-	continue;
-      case 1: /* fg */
-	string = UberButton->c->fore;
-	is_string = True;
-	break;
-      case 2: /* bg */
-	string = UberButton->c->back;
-	is_string = True;
-	break;
-      case 3: /* left */
-	val = x;
-	is_value = True;
-	break;
-      case 4: /* -left */
-	val = dpw - x - 1;
-	is_value = True;
-	break;
-      case 5: /* right */
-	val = x + w;
-	is_value = True;
-	break;
-      case 6: /* -right */
-	val = dpw - x - w - 1;
-	is_value = True;
-	break;
-      case 7: /* top */
-	val = y;
-	is_value = True;
-	break;
-      case 8: /* -top */
-	val = dph - y - 1;
-	is_value = True;
-	break;
-      case 9: /* bottom */
-	val = y + h;
-	is_value = True;
-	break;
-      case 10: /* -bottom */
-	val = dph - y - h - 1;
-	is_value = True;
-	break;
-      case 11: /* width */
-	val = w;
-	is_value = True;
-	break;
-      case 12: /* height */
-	val = h;
-	is_value = True;
-	break;
-      default: /* unknown */
-	src--;
-	continue;
-      } /* switch */
-      dest = dest_org;
-      src = --rest;
-      if (is_value)
-      {
-	if (MAX_MODULE_INPUT_TEXT_LEN - (dest - action) <= 16)
-	{
-	  /* out of space */
-	  free(action);
-	  return NULL;
-	}
-	/* print the number into the string */
-	sprintf(dest, "%d%n", val, &offset);
-	dest += offset;
-      }
-      else if (is_string)
-      {
-	if (MAX_MODULE_INPUT_TEXT_LEN - (dest - action) <= strlen(string))
-	{
-	  /* out of space */
-	  free(action);
-	  return NULL;
-	}
-	/* print the colour name into the string */
-	if (string)
-	{
-	  sprintf(dest, "%s%n", string, &offset);
-	  dest += offset;
-	}
-      }
-    } /* if */
-  } /* for */
-  *dest = 0;
-  return action;
-}
-
-void parse_window_geometry(char *geom)
+void parse_window_geometry(char *geom, int is_button_geometry)
 {
   int flags;
   int g_x;
@@ -1645,10 +1724,20 @@ void parse_window_geometry(char *geom)
   UberButton->h = 0;
   UberButton->x = 0;
   UberButton->y = 0;
-  if (flags&WidthValue)
-    w = width;
-  if (flags&HeightValue)
-    h = height;
+  if (is_button_geometry)
+  {
+    if (flags&WidthValue)
+      button_width = width;
+    if (flags&HeightValue)
+      button_height = height;
+  }
+  else
+  {
+    if (flags&WidthValue)
+      w = width;
+    if (flags&HeightValue)
+      h = height;
+  }
   if (flags&XValue)
     UberButton->x = g_x;
   if (flags&YValue)
@@ -1657,7 +1746,7 @@ void parse_window_geometry(char *geom)
     UberButton->w = 1;
   if (flags&YNegative)
     UberButton->h = 1;
-  has_button_geometry = 0;
+  has_button_geometry = is_button_geometry;
 
   return;
 }
@@ -1665,7 +1754,6 @@ void parse_window_geometry(char *geom)
 /**
 *** ParseOptions()
 **/
-extern int save_color_limit;            /* global for xpm color limiting */
 void ParseConfiguration(button_info *ub)
 {
   char *s;
@@ -1673,7 +1761,6 @@ void ParseConfiguration(button_info *ub)
   {
     NULL, /* filled out below */
     "imagepath",
-    "colorlimit",
     "colorset",
     XINERAMA_CONFIG_STRING,
     NULL
@@ -1700,14 +1787,11 @@ void ParseConfiguration(button_info *ub)
 	free(imagePath);
       CopyString(&imagePath,rest);
       break;
-    case 2:                         /* colorlimit */
-      sscanf(rest, "%d", &save_color_limit);
-      break;
-    case 3:
+    case 2:
       /* store colorset sent by fvwm */
       LoadColorset(rest);
       break;
-    case 4:
+    case 3:
       /* Xinerama state */
       FScreenConfigureModule(rest);
       break;
