@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /* This module, and the entire FvwmSaveDesktop program, and the concept for
  * interfacing this module to the Window Manager, are all original work
  * by Robert Nation and Mr. Per Persson <pp@solace.mh.se>
@@ -32,7 +33,6 @@
 #include "libs/ftime.h"
 #include <unistd.h>
 #include <ctype.h>
-#include <stdlib.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Xproto.h>
@@ -54,12 +54,12 @@ int screen;
 
 long Vx, Vy;
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      main - start of module
  *
- ***********************************************************************/
+ */
 int main(int argc, char **argv)
 {
   char *temp, *s;
@@ -113,12 +113,12 @@ int main(int argc, char **argv)
 }
 
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      Loop - wait for data to process
  *
- ***********************************************************************/
+ */
 void Loop(int *fd)
 {
     while(1) {
@@ -131,12 +131,12 @@ void Loop(int *fd)
 }
 
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      Process message - examines packet types, and takes appropriate action
  *
- ***********************************************************************/
+ */
 void process_message(unsigned long type,unsigned long *body)
 {
   switch(type)
@@ -160,12 +160,12 @@ void process_message(unsigned long type,unsigned long *body)
 
 
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      find_window - find a window in the current window list
  *
- ***********************************************************************/
+ */
 struct list *find_window(unsigned long id)
 {
   struct list *l;
@@ -183,12 +183,12 @@ struct list *find_window(unsigned long id)
 
 
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      add_window - add a new window in the current window list
  *
- ***********************************************************************/
+ */
 void add_window(unsigned long new_win, unsigned long *body)
 {
   struct list *t;
@@ -217,36 +217,37 @@ void add_window(unsigned long new_win, unsigned long *body)
 
 
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      list_new_page - capture new-page info
  *
- ***********************************************************************/
+ */
 void list_new_page(unsigned long *body)
 {
   Vx = (long)body[0];
   Vy = (long)body[1];
 }
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      SIGPIPE handler - SIGPIPE means fvwm is dying
  *
- ***********************************************************************/
-void DeadPipe(int nonsense)
+ */
+RETSIGTYPE DeadPipe(int nonsense)
 {
   exit(0);
+  SIGNAL_RETURN;
 }
 
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      writes a command line argument to file "out"
  *      checks for qoutes and stuff
  *
- ***********************************************************************/
+ */
 void write_string(FILE *out, char *line)
 {
   int len,space = 0, qoute = 0,i;
@@ -282,13 +283,13 @@ void write_string(FILE *out, char *line)
 
 
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      checks to see if we are supposed to take some action now,
  *      finds time for next action to be performed.
  *
- ***********************************************************************/
+ */
 void do_save(void)
 {
   struct list *t;
@@ -320,7 +321,7 @@ void do_save(void)
     dwidth /= t->width_inc;
     dheight /= t->height_inc;
 
-    if (IS_STICKY(t))
+    if (IS_STICKY_ACROSS_PAGES(t))
     {
       tVx = 0;
       tVy = 0;
